translations = {
    "value_error.any_str.min_length": "确保该值至少有{limit_value}个字符",
    "value_error.any_str.max_length": "确保该值最多有{limit_value}个字符",
    "value_error.any_str.length": "确保该值长度为{limit_value}个字符",
    "value_error.str.min_length": "确保该字符串至少有{limit_value}个字符",
    "value_error.str.max_length": "确保该字符串最多有{limit_value}个字符",
    "value_error.str.length": "确保该字符串长度为{limit_value}个字符",
    "value_error.str.base64": "确保该字符串为有效的Base64格式",
    "value_error.str.url": "确保该字符串为有效的URL",
    "value_error.str.email": "确保该字符串为有效的电子邮件地址",
    "value_error.str.uuid": "确保该字符串为有效的UUID",
    "value_error.str.ipv4": "确保该字符串为有效的IPv4地址",
    "value_error.str.ipv6": "确保该字符串为有效的IPv6地址",
    "value_error.str.alnum": "确保该字符串仅包含字母数字字符",
    "value_error.str.alpha": "确保该字符串仅包含字母字符",
    "value_error.str.ascii": "确保该字符串仅包含ASCII字符",
    "value_error.str.digit": "确保该字符串仅包含数字字符",
    "value_error.str.lowercase": "确保该字符串仅包含小写字母",
    "value_error.str.uppercase": "确保该字符串仅包含大写字母",
    "value_error.str.strip": "确保该字符串没有前导或尾随空白字符",
    "value_error.str.capitalize": "确保该字符串首字母大写",
    "value_error.str.title": "确保该字符串每个单词首字母大写",
    "value_error.str.replace": "确保该字符串替换成功",
    "value_error.str.slice": "确保该字符串切片成功",
    "value_error.int.convert": "确保该值可以转换为整数",
    "value_error.int.ge": "确保该值大于或等于{limit_value}",
    "value_error.int.gt": "确保该值大于{limit_value}",
    "value_error.int.le": "确保该值小于或等于{limit_value}",
    "value_error.int.lt": "确保该值小于{limit_value}",
    "value_error.int.non_negative": "确保该值为非负整数",
    "value_error.int.positive": "确保该值为正整数",
    "value_error.float.convert": "确保该值可以转换为浮点数",
    "value_error.float.ge": "确保该值大于或等于{limit_value}",
    "value_error.float.gt": "确保该值大于{limit_value}",
    "value_error.float.le": "确保该值小于或等于{limit_value}",
    "value_error.float.lt": "确保该值小于{limit_value}",
    "value_error.decimal.convert": "确保该值可以转换为十进制数",
    "value_error.datetime.invalid": "确保该值为有效的日期时间格式",
    "value_error.date.invalid": "确保该值为有效的日期格式",
    "value_error.time.invalid": "确保该值为有效的时间格式",
    "value_error.url.scheme": "确保该URL具有有效的协议方案",
    "value_error.url.host": "确保该URL具有有效的主机",
    "value_error.url.port": "确保该URL具有有效的端口",
    "value_error.url.userinfo": "确保该URL具有有效的用户信息",
    "value_error.url.query": "确保该URL具有有效的查询参数",
    "value_error.url.fragment": "确保该URL具有有效的片段标识符",
    "value_error.email.username": "确保该电子邮件地址具有有效的用户名",
    "value_error.email.domain": "确保该电子邮件地址具有有效的域名",
    "value_error.email.tld": "确保该电子邮件地址具有有效的顶级域名",
    "value_error.uuid.version": "确保该UUID具有有效的版本号",
    "value_error.uuid.format": "确保该UUID具有有效的格式",
    "value_error.ipv4": "确保该值为有效的IPv4地址",
    "value_error.ipv6": "确保该值为有效的IPv6地址",
    "value_error.any_url.scheme": "确保该URL具有有效的协议方案",
    "value_error.any_url.host": "确保该URL具有有效的主机",
    "value_error.any_url.port": "确保该URL具有有效的端口",
    "value_error.any_url.userinfo": "确保该URL具有有效的用户信息",
    "value_error.any_url.query": "确保该URL具有有效的查询参数",
    "value_error.any_url.fragment": "确保该URL具有有效的片段标识符",
    "value_error.file.allowed_extensions": "确保该文件具有有效的扩展名",
    "value_error.file.max_size": "确保该文件大小不超过{limit_value}字节",
    "value_error.file.min_size": "确保该文件大小至少为{limit_value}字节",
    "value_error.file.not_found": "确保该文件存在",
    "value_error.number.not_integer": "确保该值为整数",
    "value_error.number.not_float": "确保该值为浮点数",
    "value_error.jsondecode": "确保该值可以解码为有效的JSON",
    "type_error.none.not_allowed": "确保该值不为None",
    "type_error.none.not_none": "确保该值为None",
    "type_error.any.allow_none": "确保该值为None或者允许的类型之一",
    "type_error.any.not_none": "确保该值不为None",
    "type_error.any.invalid": "确保该值为有效的类型",
    "type_error.str": "确保该值为字符串类型",
    "type_error.bool": "确保该值为布尔类型",
    "type_error.bytes": "确保该值为字节类型",
    "type_error.bytearray": "确保该值为字节数组类型",
    "type_error.float": "确保该值为浮点数类型",
    "type_error.decimal": "确保该值为十进制数类型",
    "type_error.int": "确保该值为整数类型",
    "type_error.datetime": "确保该值为日期时间类型",
    "type_error.date": "确保该值为日期类型",
    "type_error.time": "确保该值为时间类型",
    "type_error.dict": "确保该值为字典类型",
    "type_error.list": "确保该值为列表类型",
    "type_error.tuple": "确保该值为元组类型",
    "type_error.set": "确保该值为集合类型",
    "type_error.frozenset": "确保该值为不可变集合类型",
    "type_error.enum": "确保该值为枚举类型",
    "type_error.uuid": "确保该值为UUID类型",
    "type_error.url": "确保该值为URL类型",
    "type_error.email": "确保该值为电子邮件地址类型",
    "type_error.ipv4address": "确保该值为IPv4地址类型",
    "type_error.ipv6address": "确保该值为IPv6地址类型",
    "type_error.anystr.max_length": "确保该值最多有{limit_value}个字符",
    "type_error.anystr.min_length": "确保该值至少有{limit_value}个字符",
    "type_error.anystr.length": "确保该值长度为{limit_value}个字符",
    "type_error.byteorder": "确保该值为字节顺序类型",
    "type_error.path": "确保该值为有效的路径类型",
    "value_error.missing": "确保该值不缺失",
    "value_error.extra": "确保该值不包含额外字段",
    "value_error.allow": "确保该值为允许的值之一",
    "value_error.invalid": "确保该值为有效的值",
}



from pydantic import BaseModel, ValidationError

from .translate import translate_errors


class BaseModel_i18n(BaseModel):
    def __init__(__pydantic_self__, **data) -> None:
        try:
            super().__init__(**data)
        except ValidationError as ve:
            config = __pydantic_self__.__config__
            locale = getattr(config, "locale", None)
            translations = getattr(config, "translations", None)
            if locale or translations:
                strict = getattr(config, "locale_strict", False)
                extra_translations = getattr(config, "extra_translations", None)
                translate_errors(
                    ve.errors(),
                    locale=locale,
                    translations=translations,
                    strict=strict,
                    extra_translations=extra_translations,
                )
            raise



import importlib

TRANSLATIONS = {}


def translate(
    errors: list[dict[str, str | tuple[str]]],
    translations: dict[str, str],
    strict: bool = False,
):
    if strict and not translations:
        raise ValueError("Translations must be provided when strict=True")
    if not translations and not strict:
        return
    for error in errors:
        raw, _type = error["msg"], error["type"]
        if _type in ("type_error", "value_error"):
            continue  # pragma: no cover
        msg = translations.get(_type, raw)
        if strict and msg == raw:
            raise ValueError(f"Translation for {error['type']} not found")
        ctx = error.get("ctx", None)
        if ctx:
            error["msg"] = msg.format(**ctx)
        else:
            error["msg"] = msg


def translate_errors(
    errors: list[dict[str, str | tuple[str]]],
    locale: str = "en_us",
    translations: dict[str, str] | None = None,
    strict: bool = False,
    extra_translations: dict[str, str] | None = None,
):
    if not translations:
        # Get translations by locale
        try:
            translations = TRANSLATIONS[locale]
        except KeyError:
            try:
                module = importlib.import_module(
                    f".locales.{locale}", package=__package__
                )
                translations = getattr(module, "translations", None)
                if translations:
                    TRANSLATIONS[locale] = translations
            except ModuleNotFoundError:
                translations = None

    if extra_translations:
        translations.update(extra_translations)

    translate(errors, translations, strict=strict)



import inspect

from pydantic import errors


def get_all_templates():
    """Get all templates from pydantic.errors"""

    for name, _cls in inspect.getmembers(errors):
        if (
            inspect.isclass(_cls)
            and issubclass(_cls, errors.PydanticErrorMixin)
            and _cls not in (errors.PydanticErrorMixin,)
            and not name.startswith("_")
        ):
            yield name, _cls


def get_pydantic_errors_code_templates() -> tuple[str, str | None]:
    for _, _cls in get_all_templates():
        code = getattr(_cls, "code", None) or _cls.__name__.replace("Error", "").lower()
        template = getattr(_cls, "msg_template", None)
        base_name = "type_error" if issubclass(_cls, TypeError) else "value_error"

        if base_name in ("type_error", "value_error"):
            yield base_name, template

        yield base_name + "." + code, template


def show_templates():
    from rich import print, table

    def colored(val):
        _color = "bold green" if val else "bold red"
        return f"[{_color}]{val}[/]"

    t = table.Table("Code", "Template", title="Pydantic Errors Group")
    for code, template in get_pydantic_errors_code_templates():
        t.add_row(colored(code), colored(template))
    print(t)



import importlib
from pathlib import Path

import click
from bullet import Bullet, colors
from rich import print, table

locale_dir = Path(__file__).parent / "locales"


def list_all_locales():
    for path in locale_dir.glob("*.py"):
        if path.stem == "__init__":
            continue
        module = importlib.import_module(f".locales.{path.stem}", package=__package__)
        translations = getattr(module, "translations", None)
        if translations:
            yield path.stem, translations


locale_trans = dict(list_all_locales())


cli = Bullet(
    prompt="\nPlease choose a locale to show: ",
    choices=list(locale_trans.keys()),
    indent=0,
    align=5,
    margin=2,
    shift=0,
    bullet="●",
    bullet_color=colors.foreground["magenta"],
    word_color=colors.foreground["red"],
    word_on_switch=colors.foreground["green"],
    background_color=colors.background["cyan"],
    background_on_switch=colors.background["yellow"],
    pad_right=5,
)


@click.command()
@click.option("-l", "--list_locales", is_flag=True, help="List available locales")
@click.option("-t", "--show_templates", is_flag=True, help="Show templates")
@click.option(
    "-g",
    "--generate_templates",
    is_flag=True,
    help="Generate locales using translation engine",
)
def main(list_locales: bool, show_templates: bool, generate_templates: bool):
    if show_templates:
        locale_to_show = cli.launch()
        t = table.Table("Code", "Msg template", title=f"Templates for {locale_to_show}")
        for code, msg in locale_trans[locale_to_show].items():
            t.add_row(code, msg)
        print(t)
        exit(0)
    if list_locales:
        for idx, _locale in enumerate(locale_trans):
            print(f"[light_green bold]  {idx + 1}: \[{_locale}]")
        exit(0)

    if generate_templates:
        pass


if __name__ == "__main__":
    main()



import pydantic
import pytest

from src import i18n
from src.i18n.locales.zh_cn import translations as translations_zh_cn

_TEST_TRANSLATIONS = translations_zh_cn

_VE = pydantic.ValidationError


def test_missing():
    class TestMissing(i18n.BaseModel_i18n):
        name: str

        class Config:
            locale = "zh_cn"
            locale_strict = True

    with pytest.raises(_VE, match=_TEST_TRANSLATIONS["value_error.missing"]):
        TestMissing()


def test_translations():
    class TestMissing(i18n.BaseModel_i18n):
        name: str

        class Config:
            locale_strict = True
            translations = {
                "value_error.missing": "Missing value",
            }

    with pytest.raises(_VE, match="Missing value"):
        TestMissing()


def test_extra_trans():
    class TestMissing(i18n.BaseModel_i18n):
        name: str

        class Config:
            locale_strict = True
            locale = "zh_cn"
            extra_translations = {
                "value_error.missing": "Missing value Extra",
            }

    with pytest.raises(_VE, match="Missing value Extra"):
        TestMissing()


def test_templates():
    from src.i18n.templates import show_templates

    show_templates()


def test_strict():
    class TestMissing(i18n.BaseModel_i18n):
        name: str

        class Config:
            locale_strict = True
            locale = "non-exists"

    with pytest.raises(
        ValueError, match="Translations must be provided when strict=True"
    ):
        TestMissing()

    class TestMissing(i18n.BaseModel_i18n):
        name: str

        class Config:
            locale = "non-exists"

    with pytest.raises(_VE):
        TestMissing()

    class TestMissing(i18n.BaseModel_i18n):
        name: str

    with pytest.raises(_VE, match="field required"):
        TestMissing()

    class TestMissing(i18n.BaseModel_i18n):
        name: str

        class Config:
            locale = "zh_cn"
            locale_strict = True
            extra_translations = {
                "value_error.missing": "field required",
            }

    with pytest.raises(
        ValueError, match="Translation for value_error.missing not found"
    ):
        TestMissing()


def test_ctx():
    class TestCtx(i18n.BaseModel_i18n):
        name: str

        class Config:
            locale = "zh_cn"

            min_anystr_length = 10

    with pytest.raises(
        _VE,
        match=_TEST_TRANSLATIONS["value_error.any_str.min_length"].format(
            **{"limit_value": 10}
        ),
    ):
        TestCtx(name="123")
